unit HListBox;
{ enthlt
   - THistoryList, verwendbar zur Verwaltung
     von geladenen Dateien und zum Anhngen an ein Men
   - THistoryCombo, die Combobox, die diese
     die Eingaben automatisch in einer Liste sammelt

   Autor: Elmar Warken, 1995

   erweitert: J. Rathlev, Jun. 1996
   - THistoryCombo.AddItem hinzugefgt
   - Separator in THistoryList.AddMenuItems
   - FList (hnlich FMenu) hinzugefgt zur Verwaltung der
     HistoryList in den Dialog-Komponenten
   - THistoryCombo.AddItemObject hinzugefgt, DoExit gendert }

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, IniFiles, Menus;

// Definiere CompPalPage (siehe Register)
{$Include UserComps.pas }

type
  { Event, das bei Anwahl eines der automatisch zum Men
    hinzugefgten Eintrge ausgelst wird: }
  TListMenuEvent = procedure(Sender : TObject; MenuText : string)
                     of object;

(* ----------------------------------------------------------------------- *)
{ Stringliste, die
  - sich selbst an ein Men anhngen kann, wobei sie
    die Menpunkte mit Zahlen durchnumeriert,
  - sich in eine INI-Datei speichern
    und von dort wieder lesen kann }
  THistoryList = class(TStringList)
  private
    FMenu : TMenuItem;
    FList : TStrings;
    ListSizeBefore,
    MenuSizeBefore : integer;
    { Speichert die vorherige Mengre zur spteren
      Rekonstruktion }
    FOnAutoItemClick : TListMenuEvent;
    FRadioMenu : boolean;
    FCheckedItem : string;

    procedure SetMenu(Menu : TMenuItem);
    procedure SetList(List : TStrings);
    procedure AddMenuItems;
    procedure RemoveMenuItems;
    procedure AddListItems;
    procedure RemoveListItems;
    procedure SetRadioMenu (RmOn : boolean);

  protected
    FMaxLen : integer;
    procedure SetMaxLen(AnInt : integer);
    procedure DoAutoItemClick(Sender : TObject); virtual;
       { empfngt die OnClick-Ereignisse }

  public
    constructor Create;
    { Maximale Lnge der Liste, Voreinstellung 10
      (wird nicht gespeichert, daher kein default-Teil): }
    property MaxLen : integer read FMaxLen write SetMaxLen;

    { Aufnahme eines Strings in die Liste, doppelte
      werden gelscht: }
    procedure AddString(s : string);
    procedure AddStringObject(s : string; AObject: TObject);

    { String aus der Liste entfernen }
    procedure RemString(s : string);

    (* Meneintrag auswhlen
       Ergebnis = true - Eintrag vorhanden, = false - Eintrag nicht vorhanden *) 
    function SelectMenuItem (s : string) : boolean;

    { Funktionsbereich INI-Datei: die beiden folgenden
      Funktionen mssen manuell aufgerufen werden
      (fr die Automatisierung gibt es die THistoryCombo): }
    procedure SaveToIni(IniName, IniSection : string;
                        Erase : boolean);
    procedure LoadFromIni(IniName,IniSection : string);

    procedure AssignList (AList : TStrings);

    { Funktionsbereich Men: sobald Menu einmal gesetzt ist,
      pat die Liste das Men immer an. }
    property Menu : TMenuItem read FMenu write SetMenu;

    { Funktionsbereich Liste: sobald StrList einmal gesetzt ist,
      passen sich die Listen immer an. }
    property StrList : TStrings read FList write SetList;

    property OnAutoItemClick : TListMenuEvent read FOnAutoItemClick
                                            write FOnAutoItemClick;

    property RadioMenu : boolean read FRadioMenu write SetRadioMenu;

    end;

(* ----------------------------------------------------------------------- *)
{ Combobox, die beim Laden aus einem Stream automatisch
  einen in einer INI-Datei gespeicherten Zustand wiederherstellt,
  und bei Programmende die INI-Datei aktualisiert.
  Setzt voraus, da die Instanzen dieser Klasse aus einer
  Formulardatei geladen werden (anstatt dynamisch erzeugt zu
  werden). }
  THistoryCombo = class(TComboBox)
  private
    property Items;
  protected
    FIniFileName : string;
    FIniSection  : string;
    FHistoryList : THistoryList;
    FAutoUpdate  : boolean; // JR - siehe DoExit
    FEraseSection : boolean;  // JR - siehe SaveToIni

    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override; { hier wird die INI-Datei geschrieben }
    procedure DoExit; override; { hier wird die Liste aktualisiert }
    function GetMaxItems : integer;
    procedure SetMaxItems (n : integer);

  public
    procedure LoadFromIni(IniName,IniSection : string);
    procedure AssignList (AList : TStrings);
    procedure Loaded; override; { Laden der INI-Datei findet nur hier statt }
    procedure UpdateList; { Laden der HistoryList in die eigenen Items. }
    procedure AddItem (s : string); { *neu* siehe THistoryList.AddString}
    procedure AddItemObject (s : string; AObject: TObject); { *neu* siehe THistoryList.AddStringObject}
    procedure DeleteItem (s : string); { *neu* siehe THistoryList.RemString}
    property HistoryList : THistoryList read FHistoryList; { verwaltet
      die Liste der Strings (automatische Lngenbeschrnkung etc) }

  published
    { automatisches Update der Liste }
    property AutoUpdate : boolean read FAutoUpdate write FAutoUpdate default true;
    { max. Anzahl der Eintrge in der Liste }
    property MaxItems : integer read GetMaxItems write SetMaxItems;
    { Name der zu verwendenden INI-Datei: }
    property IniFileName : string read FIniFileName write FIniFileName;
    { Name des zu verwendenden INI-Datei-Abschnitts: }
    property IniSection : string read FIniSection write FIniSection;
    property EraseSection : boolean read FEraseSection write FEraseSection default false;
  end;


(* ----------------------------------------------------------------------- *)
(* Combobox mit zustzlicher Stringliste "Values" *)
  TExtComboBox = class(TComboBox)
  published
    Values : TStringList;
    constructor Create(AOwner: TComponent); override;
  end;

procedure Register;

implementation

(********** THistoryList ***********)

constructor THistoryList.Create;
begin
  inherited Create;
  FMaxLen:=15;
  FRadioMenu:=false;
  FCheckedItem:='';
  end;

procedure THistoryList.SetRadioMenu (RmOn : boolean);
begin
  FRadioMenu:=RmOn;
  end;

procedure THistoryList.SaveToIni (IniName, IniSection : string;
                                  Erase : boolean);
var
  i : integer;
  IniFile : TIniFile;
begin
  { Da IniFile.FileName nicht nachtrglich gendert werden kann,
    mu IniFile jedesmal neu initialisiert werden: }
  IniFile:=TIniFile.Create(IniName);
  try
    if Erase then IniFile.EraseSection(IniSection);
    { Falls die INI.Datei mehr Eintrge enthlt, als die aktuelle
      Liste, werden die berzhligen Eintrge nicht berschrieben. }
    for i:=0 to Count-1 do begin
      if Strings[i]=' ' then Strings[i]:='?';   // sonst ist String leer
      IniFile.WriteString(IniSection, 'History'+IntToStr(i), Strings[i]);
      end;
  finally
    IniFile.Free;
  end;
end;

procedure THistoryList.LoadFromIni (IniName,IniSection : string);
var
  i : integer;
  IniFile : TIniFile;
  s : string;
begin
  IniFile:=TIniFile.Create(IniName);
  Clear;
  try
    { TIniFile.ReadSection sieht zwar aus, als knne sie
      die Strings auf einen Schlag lesen, liest aber nur die
      Variablennamen vor dem = }
    for i:=0 to FMaxLen-1 do begin
      s:=IniFile.ReadString(IniSection, 'History'+IntToStr(i), '');
      if s='?' then s:=' ';   // sonst ist String leer
      if (s<>'') then Add(s);
    end;
  finally
    IniFile.Free;
  end;
end;

procedure THistoryList.AssignList (AList : TStrings);
var
  i : integer;
begin
 Clear;
 for i:=0 to FMaxLen-1 do if i<AList.Count then Add(AList[i]);
 end;

procedure THistoryList.SetMenu(Menu : TMenuItem);
begin
  if Assigned(FMenu) then RemoveMenuItems;
  FMenu:=Menu; { Property-zugehrige Variable setzen }
  MenuSizeBefore:=Menu.Count; { bisherige Mengre speichern }
  AddMenuItems; { ab sofort bleibt das Men aktuell }
  end;

procedure THistoryList.RemoveMenuItems;
begin
  { alle Meneintrge ab der gespeicherten Position
    MenuSizeBefore wieder entfernen: }
  FCheckedItem:='';
  if Assigned(FMenu) then with FMenu do
    while Count>MenuSizeBefore do begin
      if Items[Count-1].Checked then FCheckedItem:=Strings[Count-1-MenuSizeBefore];
      Delete(Count-1);
      end;
  end;

procedure THistoryList.AddMenuItems;
{ Gegenstck zu RemoveMenuItems }
var
  i  : integer;
  mi : TMenuItem;
  ch : boolean;
begin
  if Assigned(FMenu) then begin
    (* nderung - keine Linie, wenn Men leer *)
    if MenuSizeBefore>0 then
      FMenu.Add(NewLine); { mit einem Separator abtrennen }
    for i:=0 to Count-1 do begin
      ch:=FRadioMenu and (Strings[i]=FCheckedItem);
      mi:=NewItem(Format('&%x %s', [i+1, Strings[i]]), 0,
                ch, True, DoAutoItemClick, 0, '');
      with mi do begin
        RadioItem:=FRadioMenu;
        GroupIndex:=99;
        end;
      FMenu.Add(mi);
      end;
    { so sieht das Men aus (erste Ziffer unterstrichen):
      ...
      ----------
      1 Eintrag1
      2 Eintrag2
    }
  end;
end;

procedure THistoryList.SetList(List : TStrings);
begin
  if Assigned(FList) then RemoveListItems;
  FList:=List; { Property-zugehrige Variable setzen }
  ListSizeBefore:=List.Count; { bisherige Mengre speichern }
  AddListItems; { ab sofort bleibt das Men aktuell }
end;

procedure THistoryList.RemoveListItems;
begin
  { alle Listeneintrge ab der gespeicherten Position
    ListSizeBefore wieder entfernen: }
  if Assigned(FList) then with FList do
    while Count>ListSizeBefore do Delete(Count-1);
end;

procedure THistoryList.AddListItems;
{ Gegenstck zu RemoveListItems }
var
  i : integer;
begin
  if Assigned(FList) then begin
    for i:=0 to Count-1 do FList.Add(Strings[i]);
  end;
end;

procedure THistoryList.SetMaxLen(AnInt : integer);
begin
  if (AnInt>=1) and (AnInt<=100) then begin
    FMaxLen:=AnInt;
    while Count>FMaxLen do
      Delete(Count-1); { alle berzhligen Eintrge lschen }
  end;
end;

procedure THistoryList.AddString(s : string);
var
  OldIndex : integer;
begin
  if length(s)>0 then begin
    { String schon vorhanden? }
    OldIndex:=IndexOf(s);
    RemoveMenuItems; { Men in Ursprungszustand versetzen... }
    if OldIndex<>-1 then Delete(OldIndex); { dann das Duplikat lschen }
    Insert(0, s); { auf alle Flle an erster Stelle einfgen }
    { Maximallnge berschritten? }
    if Count>FMaxLen then Delete(Count-1); { letzten Eintrag lschen }
    AddMenuItems; { ...und alle Eintrge neu anhngen. }
    RemoveListItems; { Liste in Ursprungszustand versetzen... }
    AddListItems; { ...und alle Eintrge neu anhngen. }
    end;
  end;

procedure THistoryList.AddStringObject(s : string; AObject: TObject);
var
  OldIndex : integer;
begin
  if length(s)>0 then begin
    { String schon vorhanden? }
    OldIndex:=IndexOf(s);
    RemoveMenuItems; { Men in Ursprungszustand versetzen... }
    if OldIndex<>-1 then Delete(OldIndex); { dann das Duplikat lschen }
    InsertObject(0,s,AObject); { auf alle Flle an erster Stelle einfgen }
    { Maximallnge berschritten? }
    if Count>FMaxLen then Delete(Count-1); { letzten Eintrag lschen }
    AddMenuItems; { ...und alle Eintrge neu anhngen. }
    RemoveListItems; { Liste in Ursprungszustand versetzen... }
    AddListItems; { ...und alle Eintrge neu anhngen. }
    end;
  end;

procedure THistoryList.RemString(s : string);
var
  OldIndex : integer;
begin
  { String vorhanden? }
  OldIndex:=IndexOf(s);
  RemoveMenuItems; { Men in Ursprungszustand versetzen... }
  if OldIndex<>-1 then Delete(OldIndex); { lschen }
  AddMenuItems; { ...und alle Eintrge neu anhngen. }
  RemoveListItems; { Liste in Ursprungszustand versetzen... }
  AddListItems; { ...und alle Eintrge neu anhngen. }
  end;

function THistoryList.SelectMenuItem (s : string) : boolean;
var
  n : integer;
begin
  n:=IndexOf(s);
  if n>=0 then FMenu.Items[n+MenuSizeBefore].Checked:=true;
  Result:=n>=0;
  end;

procedure THistoryList.DoAutoItemClick(Sender : TObject);
var
  Text : String;
  Index : Integer;
begin
  if Assigned (FOnAutoItemClick) then begin
    Text:=(Sender as TMenuItem).Caption;
    (Sender as TMenuItem).Checked:=true;
    { Text sieht so aus:
      &12 MenuText
      das "&" mu bersprungen werden, dann folgt bis
      zum Leerzeichen der Index, der angeklickt wurde }
    Index:=StrToInt('$'+Copy(Text, 2, Pos(' ', Text)-2));
    { Index - 1 ergibt den Index in der internen
      Stringliste: }
    if Index>0 then
      FOnAutoItemClick(FMenu, Strings[index-1]);
  end;
end;

(********** THistoryCombo ***********)

constructor THistoryCombo.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FAutoUpdate:=true;
  FEraseSection:=false;
  FHistoryList:=THistoryList.Create;
end;

destructor THistoryCombo.Destroy;
begin
  { Automatische Speicherung, wenn die notwendigen
    Angaben gemacht sind: }
  if (FIniFileName<>'') and (FIniSection<>'') then begin
    FHistoryList.SaveToIni(FIniFileName, FIniSection,FEraseSection);
  end;
  { da FHistoryList nicht in der Komponentenliste ist,
    mu sie manuell freigegeben werden: }
  FHistoryList.Free;
  inherited Destroy;
  end;

function THistoryCombo.GetMaxItems : integer;
begin
  Result:=FHistoryList.FMaxLen;
  end;

procedure THistoryCombo.SetMaxItems (n : integer);
begin
  FHistoryList.SetMaxLen(n);
  end;

procedure THistoryCombo.AddItem (s : string);
begin
  FHistoryList.AddString(s);
  UpdateList;
  end;

procedure THistoryCombo.AddItemObject (s : string; AObject: TObject);
begin
  FHistoryList.AddStringObject(s,AObject);
  UpdateList;
  end;

procedure THistoryCombo.DeleteItem (s : string);
begin
  FHistoryList.RemString(s);
  UpdateList;
  end;

procedure THistoryCombo.UpdateList;
begin
  Items:=FHistoryList;
  end;

procedure THistoryCombo.DoExit;
{ Fokus-Wechsel -> Eintrag speichern. }
begin
  inherited DoExit;
  if FAutoUpdate then begin    // JR - nur bei AutoUpdate speichern
    FHistoryList.AddString(Text);
    UpdateList;
    end;
  end;

procedure THistoryCombo.Loaded;
begin
  inherited Loaded;
  { Automatisches INI-Datei-Lesen, wenn die notwendigen
    Angaben gemacht sind: }
  if (FIniFileName<>'') and (FIniSection<>'') then begin
    FHistoryList.LoadFromIni(FIniFileName, FIniSection);
    UpdateList;
    end;
  end;

procedure THistoryCombo.LoadFromIni(IniName, IniSection : string);
begin
  FIniFileName:=IniName;
  FIniSection:=IniSection;
  if (FIniFileName<>'') and (FIniSection<>'') then begin
    FHistoryList.LoadFromIni(FIniFileName, FIniSection);
    UpdateList;
    end;
  end;

procedure THistoryCombo.AssignList (AList : TStrings);
begin
  FHistoryList.AssignList(AList);
  UpdateList;
  end;

(* ----------------------------------------------------------------------- *)
(* Combobox mit zustzlicher Stringliste "Values" *)
constructor TExtComboBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Values := TStringList.Create;
  end;


(************** Register ***************)

procedure Register;
begin
  RegisterComponents(CompPalPage, [THistoryCombo]);
  RegisterComponents(CompPalPage, [TExtComboBox]);
end;

end.
