(* Delphi-Unit -
   verschiedene Unterroutinen
   ==========================
   - String-Konvertierungen ANSI <-> OEM
   - div. String-Routinen (z.B. Leerezeichen anfgen oder lschen)
   - Dateipfade zerlegen und zusammensetzen
   - Befehlszeilen auswerten
   - Umwandlung Grad:Min:Sek -> String und zurck
   - Substrings und Zahlen bis Begrenzer extrahieren
   - Gltigkeitsprfung fr Zahlen
   - Umwandlung Integer und Double -> String mit fhrenden Nullen und Tausendermark.
   - Dateigre in String
   - Zusammensetzen von Word und Cardinal aus Bytes
   - Typdeklarationen fr das Zerlegen von Word, LongWort und Int64 in Bytes und Words
   - schnelles Sortieren einer Liste (Quicksort)

    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Neue Zusammenstellung - Okt. 2006
   *)


unit JrUtils;

interface

uses
  Windows, SysUtils, Classes, Graphics, Forms, Controls,
  Dialogs, Messages, IniFiles, Printers;

type
  DateStr = string[10];
  TCvTable = array [0..127] of char;

  // Compare function for Quicksort
  TCompFunction = function (const arg1,arg2) : boolean of object;

  TDelimiters = set of char;

const
  ChrID = '#';             // Kennzeichner fr num. Zeichenwerte
  Strich = '-';
  Komma = ',';
  Punkt = '.';
  Semikolon = ';';
  Space = ' ';
  Tab = #9;
  DPunkt = ':';

  DezTr : char = Punkt;    // Dezimalzeichen
  ThMark : char = Space;   // Zeichen fr Tausendergruppen
  TmMark : char = DPunkt;  // Zeichen fr Uhrzeitgruppen hh:mm:ss

  xDef = 1.0;              // Standardwert ReadNxtDbl
  nDef = 1;                // Standardwert ReadNxtInt
  cDef = #0;               // Standardwert ReadNxtChr

  TabToANSI : TCvTable =(
    #$C7,#$FC,#$E9,#$E2,#$E4,#$E0,#$E5,#$E7,#$EA,#$EB,#$E9,#$E8,#$EF,#$EE,#$C4,#$C5,
    #$C9,#$E6,#$C6,#$F4,#$F6,#$F2,#$FB,#$F9,#$FF,#$D6,#$DC,#$A2,#$A3,#$A5,#$20,#$20,
    #$E1,#$ED,#$F3,#$FA,#$F1,#$D1,#$61,#$6F,#$BF,#$20,#$AC,#$BD,#$BC,#$A1,#$AB,#$BB,
    #$20,#$20,#$20,#$A6,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,
    #$20,#$20,#$20,#$20,#$AD,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,
    #$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,
    #$20,#$DF,#$B6,#$20,#$20,#$20,#$B5,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,
    #$20,#$B1,#$20,#$20,#$20,#$20,#$F7,#$20,#$B0,#$20,#$B7,#$20,#$B3,#$B2,#$20,#$20);
  TabToOEM : TCvTable =(
    #$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,
    #$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,
    #$20,#$AD,#$9B,#$9C,#$20,#$9D,#$B3,#$15,#$20,#$20,#$20,#$AE,#$AA,#$C4,#$20,#$20,
    #$F8,#$F1,#$FD,#$FC,#$20,#$E6,#$E3,#$20,#$20,#$20,#$20,#$AF,#$AC,#$AB,#$20,#$A8,
    #$41,#$41,#$41,#$41,#$8E,#$8F,#$92,#$80,#$45,#$90,#$45,#$45,#$49,#$49,#$49,#$49,
    #$44,#$A5,#$4F,#$4F,#$4F,#$4F,#$99,#$20,#$20,#$55,#$55,#$55,#$9A,#$59,#$20,#$E1,
    #$85,#$61,#$83,#$61,#$84,#$86,#$91,#$87,#$8A,#$82,#$88,#$89,#$8D,#$A1,#$8C,#$8B,
    #$20,#$A4,#$95,#$A2,#$93,#$6F,#$94,#$F6,#$20,#$97,#$A3,#$96,#$81,#$79,#$20,#$98);

  IllegalFilenameChars = [#0..#31,'*','?','<','>',':','"','/','\','|'];

  CenterPos : TPoint = (X : -1; Y : -1);
  MaxHist : integer = 50;

type
  TBoolFunction = function : boolean of object;
  TIntegerFunction = function : integer;

  TWinkelFormat = (fmDezGrad,fmDezMin,fmSekunden);

  TWord = record
    case integer of
    0 : (Both : word);
    1 : (Lo,Hi : byte);
    end;

  TLongWord = record
    case integer of
    0 : (LongWord : cardinal);
    1 : (Lo,Hi : word);
    2 : (LoL,LoH,HiL,HiH : byte);
    3 : (Bytes : array [0..3] of Byte);
    end;

  TInt64 = record
    case integer of
    0: (AsInt64 : int64);
    1: (Lo, Hi  : Cardinal);
    2: (Cardinals: array [0..1] of Cardinal);
    3: (Words: array [0..3] of Word);
    4: (Bytes: array [0..7] of Byte);
    end;

  TFontStyleToByte = record
    case integer of
    1 : (Style : TFontStyles);
    2 : (Value : byte);
    end;

  TFPoint = record
    X,Y : double;
    end;

  TFRect = record
    case integer of
    0 : (Left,Top,Right,Bottom : double);
    1 : (TopLeft,BottomRight : TFPoint);
    end;

{ ---------------------------------------------------------------- }
(* Verzgerungszeit unter Verwendung der Windows Waitable Timer Objects
   Wait: warten bis Zeit in ms abgelaufen ist (min. Zeit 100 ns) *)

  TWaitTimer = class (TObject)
    FTimer   : THandle;
    constructor Create;
    destructor Destroy;  override;
    procedure Wait (DelayTime : double);
    end;

{ ---------------------------------------------------------------- }
// Umwandeln eines DOS-Zeichens in eines Grobuchstaben (auch Umlaute) 
function OEMUpCase (c : char) : char;

// Umwandeln eines ANSI-Zeichens in eines Grobuchstaben (auch Umlaute) 
function ANSIUpCase (c : char) : char;

// Umwandeln eines DOS-Strings in Grobuchstaben (auch Umlaute) 
function OEMUpString (const S : string) : string;

// Umwandeln eines ANSI-Strings in Grobuchstaben (auch Umlaute) 
function ANSIUpString (const S : string) : string;

// Umwandeln eines Strings in Grobuchstaben (Umlaute in AE, OE, UE) zu Sortierzwecken
function OEMUpSort (const S : string) : string;

// Umwandeln eines Zeichens in Kleinbuchstaben
function LowCase(Ch: Char): Char;

// Umwandeln eines OEM-Strings nach ANSI
function StrToAnsi (const s : string) : string;

// Umwandeln eines ANSI-Strings nach OEM 
function StrToOEM (const s : string) : string;

// Substring-Pos. ohne Unterschreidung gro/klein
function TextPos (const Substr, S: string): Integer;
function TextPosEx (const Substr, S: string; Offset : cardinal): Integer;

{ ---------------------------------------------------------------- }
// String mit Leerstellen erzeugen
function FillSpace (len : integer) : string;

// fhrende Leerstellen ergnzen
function AddSp (const S : string;
                len     : integer) : string;

// Leerstellen am Ende ergnzen
function ExtSp (const S : string;
                len   : integer) : string;

// fhrende Leerstellen lschen
function RemSp (const S : string) : string;

// Leerstellen am Ende lschen
function CutSp (const S : string) : string;

// alle Leerstellen im String lschen
function DelSp (const S : string) : string;

// mehrfache Leerstellen im String lschen
function DelMulSp (const S : string) : string;

// Steuerzeichen im String lschen bzw. ersetzen
function DelCtrlChars(const S : string) : string;
function CrLfToChar (s : string; c : char) : string;
function CharToCr (s : string; c : char) : string;

// erstes nichtleeres Zeichen suchen
function SeekNoSp (const s : string) : integer;

// nchstes Leerzeichen oder Tab suchen
function SeekSpace (const s : string) : integer;

// String mit Zeichen erzeugen
function FillStr (c   : char;
                  len : integer) : string;

// fhrende Zeichen ergnzen
function AddChar (const S : string;
                  c       : char;
                  len     : integer) : string;

// Zeichen am Ende ergnzen
function ExtChar (const S : string;
                  c       : char;
                  len     : integer) : string;

// Leerzeichen gegen _ und / gegen % austauschen oder zurck
function ReplOptionChar (s : string; Back : boolean) : string;

// String auf feste Lnge bringen (krzen oder mit Leerzeichen auffllen)
function FixLength (const S : string;
                    len     : integer) : string;

// Anzahl von SubStr in S zhlen
function CharCount (const SubStr,S : string) : integer;

{ ---------------------------------------------------------------- }
// Ausmaskieren der Erweiterung 
function GetExt (const Name : string) : string;

// Dateinamenerweiterung entfernen 
function DelExt (Name : string) : string;

// ndern einer Dateinamenerweiterung 
function NewExt (Name,Ext  : string) : string;

// Dateiname mit Laufwerk, Verzeichnis und Extension (falls nicht vorhanden) erweitern
function Erweiter (Pfad,Name,Ext : string) : string;

// Suffix zu Dateinamen hinzufgen 
function AddNameSuffix (FName,Suffix,Ext : string) : string;

// Suffix zu Dateinamen mit Extension hinzufgen
function InsertNameSuffix (FName,Suffix : string) : string;

// Pfad aus Dateinamen teilweise entfernen, wenn der Name zu lang wird 
function StripPath (Name : String;
                    Len  : integer) : String;

// Bei Verzeichnisangaben Backslash ergnzen 
function SetDirName (Dir : string) : string;

// Exrahiere den bergeordneten Pfad 
function ExtractParentPath (Dir : string) : string;

// kein spezielles Verzeichnis (selbst und eins zurck) 
function NotSpecialDir (const Name : string) : boolean;

// enthlt vollstndigen Pfad 
function ContainsFullPath (const Name : string) : boolean;

// prfe, ob alle Zeichen in Filename zulssig sind 
function CheckFilename (const Filename : string) : boolean;

{ ---------------------------------------------------------------- }
// Routinen zur Auswertung einer Befehlszeile
// prfe, ob die ersten Zeichen einer Option mit dem Parameter bereinstimmen
function CompareOption (Param,Option : string) : boolean;

// Option vom Typ /option:value einlesen
function ReadOptionValue (var Param : string; Option : string) : boolean;

{ ---------------------------------------------------------------- }
(* Integer, Double oder String aus einem String s bis zum nchsten
   Begrenzer (Del) lesen
   s wird um den verarbeiteten Teil gekrzt *)
function ReadNxtInt (var s   : String;
                     Del     : char;
                     Default : integer;
                     var err : boolean) : integer; overload;

function ReadNxtInt (var s   : String;
                     Del     : char;
                     Default : integer) : integer; overload;

function ReadNxtDbl (var s   : String;
                     Del     : char;
                     Default : double;
                     var err : boolean) : double; overload;

function ReadNxtDbl (var s   : String;
                     Del     : char;
                     Default : double) : double; overload;

function ReadNxtStr (var s   : String;
                     Del     : char) : string;

function ReadNxtWord (var s : string;
                      Dels  : TDelimiters) : string;

function ReadNxtChr (var s   : String;
                     Del     : char;
                     Default : char;
                     var err : boolean) : char; overload;

function ReadNxtChr (var s   : String;
                     Del     : char;
                     Default : char) : char; overload;

{ ---------------------------------------------------------------- }
(* Testen, ob eine gltige Integer oder Double Zahl in einem String s
   bis zum nchsten Begrenzer (Del) steht *)
function TestNxtNumber (s       : String;
                        Del     : char ) : boolean;


// Wert auf Gltigkeit prfen - Result=true : x <> NoVal 
function Valid (x,NoVal : double) : boolean;

{ ---------------------------------------------------------------- }
// Dezimaltrenner setzen und lesen, Standard = Punkt 
procedure SetDecPtChar (Zeichen : char);
function GetDecPtChar : char;

// Tausendermarkierung, Standard = Leerzeichen 
procedure ThMarkChar (Zeichen : char);

// Integer-Zahl in String umsetzen 
function StrInt (x : int64;
                 n : integer) : string;

// Integer-Zahl in String mit fhrenden Nullen umsetzen 
function ZStrInt (x : int64;
                  n : integer) : string;

// Double-Zahl in String umsetzen 
function StrDbl (x   : double;
                 n,d : integer) : string;

// Double-Zahl in String mit fhrenden Nullen umsetzen
function ZStrDbl (x   : double;
                  n,d : integer) : string;

// Double-Zahl in String in Dreiergruppen umsetzen
function NStrDbl (x : double;
                  n,d : integer) : string;

// Bytes, je nach Gre in String umsetzen 
function SizeToStr (Bytes : int64) : string;

{ ---------------------------------------------------------------- }
// Dezimalkomma ersetzen 
function ReplDecPt (s : string) : string;

// Dezimalpunkt einsetzen 
function SetDecPt (s : string) : string;

{ ---------------------------------------------------------------- }
// Cardinal in Hi- oder Lo-Wort zerlegen 
function HiWord (n : cardinal) : word;

function LoWord (n : cardinal) : word;

// Cardinal aus Hi- und Lo-Wort zusammensetzen 
function JoinWord (HiWord,LoWord : word) : cardinal;

// Cardinal aus 4 Bytes zusammensetzen 
function JoinByte (HHByte,HLByte,LHByte,LLByte : word) : cardinal;

// Convert 16-bit oder 8-Bit number to hex string
function HexByte (b : byte) : string;

function HexWord (w : word) : string;

{ ---------------------------------------------------------------- }
// schnelles Sortieren einer Liste
procedure QuickSort (var SList;
                     Count,RSize : Integer;
                     Compare     : TCompFunction);

{ ---------------------------------------------------------------- }
// erweiterte Drucker-Angaben (alle Angaben in mm)
function GetPaperWidth (APrinter : TPrinter) : integer;
function GetPaperHeight (APrinter : TPrinter) : integer;
function GetLeftOffset (APrinter : TPrinter) : integer;
function GetTopOffset (APrinter : TPrinter) : integer;
function GetMaxWidth (APrinter : TPrinter) : integer;
function GetMaxHeight (APrinter : TPrinter) : integer;

{ ---------------------------------------------------------------- }
// Prfen, ob ein Fenster auf den Bildschirm passt
procedure CheckScreenBounds (Screen         : TScreen;
                             var ALeft,ATop : integer;
                             AWidth,AHeight : integer);

{ ---------------------------------------------------------------- }
// MessageDlg in Bildschirmmitte (X<0) oder an Position X,Y
function MessageDialog(const Title,Msg: string; DlgType: TMsgDlgType;
                       Buttons: TMsgDlgButtons; Pos : TPoint) : integer;

function ConfirmDialog (const Title,Msg : string; Pos : TPoint) : boolean;

procedure InfoDialog (const Title,Msg : string; Pos : TPoint);

procedure ErrorDialog (const Title,Msg : string; Pos : TPoint);

{ ---------------------------------------------------------------- }
// get current cursor position
function CursorPos : TPoint;
function CursorOffsetPos (Ofs : TPoint): TPoint;

{ ---------------------------------------------------------------- }
// History list management
procedure LoadHistory (IniFile : TIniFile; Section : string; History : TStrings);
procedure SaveHistory (IniFile : TIniFile; Section : string; History : TStrings);
procedure AddToHistory (History : TStrings; hs : string);
procedure RemoveFromHistory (History : TStrings; hs : string);

{ ---------------------------------------------------------------- }
// Entferne alle Objekte einer String-Liste aus dem Speicher
procedure FreeListObjects(Liste : TStrings);

{ ---------------------------------------------------------------- }
// Tastaturpuffer lschen
function ClearKeyboardBuffer : Integer;

{ ---------------------------------------------------------------- }
implementation

uses StrUtils;

{ ---------------------------------------------------------------- }
(* Umwandeln eines Zeichens in eines Grobuchstaben (auch Umlaute)
   OEM-Zeichen *)
function OEMUpCase (c : char) : char;
begin
  if c<#127 then Result:=upcase(c)
  else begin
    case c of
    '' : Result:='';
    '' : Result:='';
    '' : Result:='';
    '' : Result:='S';
      else Result:=c;
      end;
    end;
  end;

{ ---------------------------------------------------------------- }
(* Umwandeln eines Zeichens in eines Grobuchstaben (auch Umlaute)
   ANSI-Zeichen *)
function ANSIUpCase (c : char) : char;
begin
  if c<#127 then Result:=upcase(c)
  else begin
    case c of
    '' : Result:='';
    '' : Result:='';
    '' : Result:='';
    '' : Result:='S';
      else Result:=c;
      end;
    end;
  end;

{ ---------------------------------------------------------------- }
(* Umwandeln eines Strings in Grobuchstaben (auch Umlaute) *)
function OEMUpString (const S : string) : string;
var
  i  : integer;
  ns : string;
  c  : char;
begin
  ns:='';
  for i:=1 to length(s) do begin
    c:=s[i];
    if c<#127 then ns:=ns+upcase(c)
    else begin
      case c of
      '' : ns:=ns+'';
      '' : ns:=ns+'';
      '' : ns:=ns+'';
      '' : ns:=ns+'SS';
        else ns:=ns+c;
        end;
      end;
    end;
  OEMUpString:=ns;
  end;

{ ---------------------------------------------------------------- }
(* Umwandeln eines ANSI-Strings in Grobuchstaben (auch Umlaute) *)
function ANSIUpString (const S : string) : string;
var
  i  : integer;
  ns : string;
  c  : char;
begin
  ns:='';
  for i:=1 to length(s) do begin
    c:=s[i];
    if c<#127 then ns:=ns+upcase(c)
    else begin
      case c of
      '' : ns:=ns+'';
      '' : ns:=ns+'';
      '' : ns:=ns+'';
      '' : ns:=ns+'SS';
        else ns:=ns+c;
        end;
      end;
    end;
  ANSIUpString:=ns;
  end;

{ ---------------------------------------------------------------- }
(* Umwandeln eines Strings in Grobuchstaben (Umlaute in AE, OE, UE)
   zu Sortierzwecken *)
function OEMUpSort (const S : string) : string;
var
  i  : integer;
  ns : string;
  c  : char;
begin
  ns:='';
  for i:=1 to length(s) do begin
    c:=s[i];
    if c<#127 then ns:=ns+upcase(c)
    else begin
      case c of
      '','' : ns:=ns+'AE';
      '','' : ns:=ns+'OE';
      '','' : ns:=ns+'UE';
      ''     : ns:=ns+'SS';
        else ns:=ns+c;
        end;
      end;
    end;
  OEMUpSort:=ns;
  end;

{ ---------------------------------------------------------------- }
// Umwandeln eines Zeichens in Kleinbuchstaben
function LowCase(Ch: Char): Char;
begin
  Result:=Ch;
  if (Ch>='A') and (Ch<='Z') then inc(Result,Ord('a') - Ord('A'));
  end;

{ ---------------------------------------------------------------- }
(* Umwandeln eines OEM-Strings nach ANSI *)
function StrToAnsi (const s : string) : string;
var
  i  : integer;
begin
  Result:=s;
  for i:=1 to length(Result) do begin
    if Result[i]>=#128 then Result[i]:=TabToANSI[ord(Result[i])-128]
    else if Result[i]=#$15 then Result[i]:=#$A7;
    end;
  end;

{ ---------------------------------------------------------------- }
(* Umwandeln eines ANSI-Strings nach OEM *)
function StrToOEM (const s : string) : string;
var
  i  : integer;
begin
  Result:=s;
  for i:=1 to length(Result) do begin
    if Result[i]>=#128 then Result[i]:=TabToOEM[ord(Result[i])-128];
    end;
  end;

{------------------------------------------------------------------}
(* Substring-Pos. ohne Unterschreidung gro/klein *)
function TextPos (const Substr, S: string): Integer;
begin
  Result:=Pos (AnsiLowerCase(Substr),AnsiLowerCase(s));
  end;

function TextPosEx (const Substr, S: string; Offset : cardinal): Integer;
begin
  Result:=PosEx (AnsiLowerCase(Substr),AnsiLowerCase(s),Offset);
  end;

{------------------------------------------------------------------}
(* String mit Leerstellen erzeugen *)
function FillSpace (len : integer) : string;
var
  i : byte;
  s : string;
begin
  if len>255 then len:=255;
  s:='';
  for i:=1 to len do s:=s+' ';
  FillSpace:=s;
  end;

{------------------------------------------------------------------}
(* fhrende Leerstellen ergnzen *)
function AddSp (const S : string;
                len     : integer) : string;
var
  i  : integer;
begin
  Result:=s;
  for i:=succ(length(Result)) to len do Result:=' '+Result;
  end;

{------------------------------------------------------------------}
(* Leerstellen am Ende ergnzen *)
function ExtSp (const S : string;
                len     : integer) : string;
var
  i  : integer;
begin
  Result:=s;
  for i:=succ(length(Result)) to len do Result:=Result+' ';
  end;

{------------------------------------------------------------------}
(* fhrende Leerstellen und Tabs lschen *)
function RemSp (const S : string) : string;
var
  i : integer;
begin
  Result:=s; i:=1;
  while (i<=length(Result)) and ((Result[i]=Space) or (Result[i]=Tab)) do inc(i);
  delete(Result,1,pred(i));
  end;

{ --------------------------------------------------------------- }
(* Leerstellen am Ende lschen *)
function CutSp (const S : string) : string;
var
  i : integer;
begin
  Result:=s; i:=length(Result);
  while (i>0) and (Result[i]=Space) do dec(i);
  delete(Result,succ(i),length(Result));
  end;

{ --------------------------------------------------------------- }
(* alle Leerstellen im String lschen *)
function DelSp (const S : string) : string;
var
  i  : integer;
begin
  Result:=s; i:=1;
  while (i<=length(Result)) do begin
    if Result[i]=Space then delete(Result,i,1) else inc(i);
    end;
  end;

{ --------------------------------------------------------------- }
(* mehrfache Leerstellen im String lschen *)
function DelMulSp (const S : string) : string;
var
  i  : integer;
begin
  Result:=s;
  if length(Result)>0 then begin
    i:=1;
    while (i<length(Result)) do begin
      if (Result[i]=Space) and (Result[i+1]=Space) then delete(Result,i,1) else inc(i);
      end;
    if (Result[i]=Space) then delete(Result,i,1);
    end;
  end;

{ --------------------------------------------------------------- }
(* Steuerzeichen im String lschen *)
function DelCtrlChars(const S : string) : string;
var
  i  : integer;
begin
  Result:=s;
  if length(Result)>0 then begin
    i:=1;
    while (i<length(Result)) do begin
      if Result[i]<#32 then delete(Result,i,1) else inc(i);
      end;
    end;
  end;

// Cr-Lf im String durch spez. Zeichen ersetzen
function CrLfToChar (s : string; c : char) : string;
begin
  Result:=DelCtrlChars(AnsiReplaceStr(s,#$0D,c));
  end;

// spez. Zeichen im String durch Cr-Lf ersetzen
function CharToCr (s : string; c : char) : string;
begin
  Result:=AnsiReplaceStr(s,c,#$0D);
  end;

{ --------------------------------------------------------------- }
(* erstes nichtleeres Zeichen suchen *)
function SeekNoSp (const s : string) : integer;
var
  i  : byte;
begin
  i:=1;
  while (i<=length(s)) and (s[i]=Space) do inc(i);
  if i<=length(s) then SeekNoSp:=i else SeekNoSp:=0;
  end;

{ --------------------------------------------------------------- }
(* Position fr nchstes Leerzeichen oder Tab suchen,
   bei Stringende = Lnge+1 *)
function SeekSpace (const s : string) : integer;
var
  i  : byte;
begin
  if length(s)=0 then Result:=0
  else begin
    i:=1;
    while (i<=length(s)) and (s[i]<>Space) and (s[i]<>Tab) do inc(i);
    Result:=i;
    end;
  end;

{ --------------------------------------------------------------- }
(* String mit Zeichen erzeugen *)
function FillStr (c   : char;
                  len : integer) : string;
var
  i : byte;
  s : string;
begin
  if len>255 then len:=255;
  s:='';
  for i:=1 to len do s:=s+c;
  FillStr:=s;
  end;

{ --------------------------------------------------------------- }
(* fhrende Zeichen ergnzen *)
function AddChar (const S : string;
                  c       : char;
                  len     : integer) : string;
var
  i  : integer;
begin
  Result:=s;
  for i:=succ(length(Result)) to len do Result:=c+Result;
  end;

{ --------------------------------------------------------------- }
(* Zeichen am Ende ergnzen *)
function ExtChar (const S : string;
                  c       : char;
                  len     : integer) : string;
var
  i  : integer;
begin
  Result:=s;
  for i:=succ(length(Result)) to len do Result:=Result+c;
  end;

{ --------------------------------------------------------------- }
(* Leerzeichen gegen _ und / gegen % austauschen oder zurck *)
function ReplOptionChar (s : string; Back : boolean) : string;
var
  j : integer;
begin
  if Back then begin
    for j:=1 to length(s) do begin
      if s[j]='_' then s[j]:=' ';
      if s[j]='%' then s[j]:='/';
      end;
    end
  else begin
    for j:=1 to length(s) do begin
      if s[j]=' ' then s[j]:='_';
      if s[j]='/' then s[j]:='%';
      end;
    end;
  Result:=s;  
  end;

{ --------------------------------------------------------------- }
(* String auf feste Lnge bringen (krzen oder mit Leerzeichen auffllen *)
function FixLength (const S : string;
                    len : integer) : string;
begin
  if length(s)<len then FixLength:=ExtSp (S,len)
  else FixLength:=copy (s,1,len);
  end;

{ --------------------------------------------------------------- }
(* Anzahl von SubStr in S zhlen *)
function CharCount (const SubStr,S : string) : integer;
var
  k,n : integer;
begin
  Result:=0; n:=1;
  while (n<length(S)) do begin
    k:=PosEx(SubStr,S,n);
    if k>0 then begin
      inc(Result);
      n:=k+1;
      end
    else n:=length(s);
    end;
  end;

{ --------------------------------------------------------------- }
(* Ausmaskieren der Erweiterung (ohne Punkt) *)
function GetExt (const name : string) : string;
var
  i,j : integer;
begin
  j:=length(Name);
  if j>0 then begin
    i:=j;
    while (i>0) and (Name[i]<>'.') do dec(i);
    if i=0 then Result:=''
    else Result:=copy(name,i+1,j-i);
    end
  else Result:='';
  end;

{ --------------------------------------------------------------- }
(* Dateinamenerweiterung entfernen (sucht letzten Punkt) *)
function DelExt (Name : string) : string;
var
  i,j : integer;
begin
  j:=length(Name);
  if j>0 then begin
    i:=j;
    while (i>0) and (Name[i]<>'.') do dec(i);
    if i>0 then delete (Name,i,j-i+1);
    end;
  Result:=Name;
  end;

{ --------------------------------------------------------------- }
(* ndern einer Dateinamenerweiterung (sucht letzten Punkt - JR 9.8.05) *)
function NewExt (Name,Ext  : string) : string;
begin
  Name:=DelExt(Name);
  if (length(Ext)>0) and (Ext[1]='.') then delete(Ext,1,1);
  if length(Ext)>0 then Result:=Name+'.'+Ext
  else Result:=Name;
  end;

{ --------------------------------------------------------------- }
(* Dateiname mit Laufwerk, Verzeichnis und
   Extension (falls nicht vorhanden) erweitern *)
function Erweiter (Pfad,Name,Ext  : string) : string;
var
  stemp : string;
begin
  if (length(Ext)>0) and (Ext[1]='.') then delete(Ext,1,1);
  if (pos('.',Name)=0) and (length(Ext)<>0) then Name:=Name+'.'+ext;
  if (pos (':',Name)<>0) or (pos('\',Name)<>0)
     or (length(Pfad)=0) then stemp:=Name
  else if Pfad[length(Pfad)]='\' then stemp:=Pfad+Name
         else stemp:=Pfad+'\'+Name;
  Result:=stemp;
  end;

{ ---------------------------------------------------------------- }
(* Suffix und Erweiterung zu Dateinamen hinzufgen *)
function AddNameSuffix (FName,Suffix,Ext : string) : string;
begin
  if (length(Ext)>0) and (Ext[1]<>'.') then Ext:='.'+Ext;
  Result:=DelExt(FName)+Suffix+Ext;
  end;

{ ---------------------------------------------------------------- }
(* Suffix zu Dateinamen mit Extension hinzufgen *)
function InsertNameSuffix (FName,Suffix : string) : string;
begin
  Result:=AddnameSuffix(FName,Suffix,GetExt(FName));
  end;

{ ---------------------------------------------------------------- }
(* Pfad aus Dateinamen teilweise entfernen, wenn der Name zu lang wird
  siehe auch MinimizeName ind FileCtrl *)
function StripPath (Name : String;
                    Len  : integer) : String;
const
  Punkte = '...';
var
  i,j,nl : integer;
  ok     : boolean;
begin
  nl:=length(Name);
  if nl>=Len then begin
    i:=nl; ok:=true;
    if IsPathDelimiter(Name,i) then dec(i);
    while not IsPathDelimiter(Name,i) and (i>0) do dec(i);
    if i=0 then Name:=Punkte+copy(Name,nl-Len+3,nl)
    else begin
      dec(i); j:=i;
      repeat
        while not IsPathDelimiter(Name,j) and (j>0) do dec(j);
        dec(j);
        if j<0 then Name:=Punkte+copy(Name,nl-Len+3,nl)
        else begin
          ok:=nl-i+j+4<=Len;
          end;
        until ok or (j<0);
      if ok then begin
        inc(j,2);
        delete (Name,j,succ(i-j)); insert (Punkte,Name,j);
        end;
      end;
    end;
  Result:=Name;
  end;

{ ------------------------------------------------------------------- }
(* Exrahiere den bergeordneten Pfad      *)
function ExtractParentPath (Dir : string) : string;
var
  i : integer;
begin
  Dir:=ExcludeTrailingPathDelimiter(Dir);
  i:=length(Dir);
  while not IsPathDelimiter(Dir,i) and (i>0) do dec(i);
  if i<=1 then Result:=Dir
  else Result:=copy(Dir,1,i-1);
  end;

{ ------------------------------------------------------------------- }
(* Bei Verzeichnisangaben Backslash ergnzen *)
function SetDirName (Dir : string) : string;
begin
  if length(Dir)>0 then Result:=IncludeTrailingPathDelimiter(Dir)
  else Result:='';
//  if (length(Dir)=0) or (Dir[length(Dir)]='\') then Result:=Dir
//  else Result:=Dir+'\';
  end;

{ ------------------------------------------------------------------- }
(* kein spezielles Verzeichnis (selbst und eins zurck) *)
function NotSpecialDir (const Name : string) : boolean;
begin
  Result:=(Name<>'.') and (Name<>'..');
  end;

{ ------------------------------------------------------------------- }
(* Name enthlt vollstndigen Pfad *)
function ContainsFullPath (const Name : string) : boolean;
begin
  if length(Name)>0 then Result:=(Name[1]='\') or (pos(':',Name)>0)
  else Result:=false;
  end;

{ ---------------------------------------------------------------- }
(* prfe, ob alle Zeichen in Filename zulssig sind *)
function CheckFilename (const Filename : string) : boolean;
var
  i : integer;
begin
  Result:=true; i:=1;
  if length(Filename)>0 then while Result and (i<=length(Filename)) do begin
    Result:=not (Filename[i] in IllegalFilenameChars);
    inc(i);
    end;
  end;

{ ---------------------------------------------------------------- }
// Routinen zur Auswertung einer Befehlszeile
// prfe, ob die ersten Zeichen einer Option mit dem Parameter bereinstimmen
function CompareOption (Param,Option : string) : boolean;
begin
  Result:=AnsiLowercase(Param)=copy(Option,1,length(Param));
  end;

// Option vom Typ option:value einlesen
function ReadOptionValue (var Param : string; Option : string) : boolean;
var
  i : integer;
begin
  Result:=false;
  i:=AnsiPos(':',Param);
  if i=0 then exit;
  if not CompareOption(copy(Param,1,i-1),Option) then exit;
  Delete(Param,1,i); Result:=true;
  end;

{------------------------------------------------------------------}
(* Winkel-Wert konvertieren *)
function StrToDeg (s     : string;
                   var v : double) : boolean;
var
 t      : string;
 x,y    : double;
 j,k    : integer;
begin
  j:=0;
  k:=pos(',',s); if k>0 then s[k]:='.';
  if length(s)<>0 then begin
    k:=pos(':',s);
    if k=0 then k:=pos('/',s);
    if k=0 then k:=pos('"',s);
    if k=0 then val(s,v,j)
    else begin (* Grad:Min *)
      val (copy(s,1,pred(k)),x,j); (* Grad *)
      if j=0 then begin
        t:=copy (s,succ(k),255);
        k:=pos(':',t);
        if k=0 then k:=pos('/',t);
        if k=0 then k:=pos(#$27,s);
        if k=0 then begin (* dez.Min *)
          if length(t)>0 then begin
            val (t,v,j);
            if (j=0) and (v<60.0) then begin
              if x>=0.0 then v:=x+v/60.0
              else v:=x-v/60.0;
              end
            else j:=1;
            end
          else v:=x;
          end
        else begin (* Sek. *)
          val (copy (t,1,pred(k)),y,j);  (* Min *)
          if (j=0) and (y<60.0) then begin
            t:=copy (t,succ(k),255);
            if length(t)>0 then begin
              val (t,v,j); (* Sek. *)
              if (j=0) and (v<60.0) then begin
                if x>=0.0 then v:=x+(y+v/60.0)/60.0
                else v:=x-(y-v/60.0)/60.0;
                end
              else j:=1;
              end
            else begin
              if x>=0.0 then v:=x+y/60.0
              else v:=x-y/60.0;
              end;
            end
          else j:=1;
          end;
        end;
      end;
    end
  else v:=0.0;
  StrToDeg:=j=0;
  end;

{------------------------------------------------------------------}
(* Winkel in String mit vorgebenem Format umsetzen
   Stellen: w : gesamt
            d : hinter Komma (einschl. Minuten und Sekunden
   Typ : m = fmDezGrad  : Grad,xxx
             fmDezMin   : Grad:Minuten,xx
             fmSekunden : Grad:Minuten:Sekunden,xxx *)
function DegToStr(v      : double;
                  Format : TWinkelFormat;
                  w,d    : integer) : string;
var
  s,t   : string;
  x,y,z : extended;
  i     : integer;
begin
  x:=int(abs(v)); y:=60.0*frac(abs(v));
  case Format of
  fmDezGrad : str(v:w:d,s);
  fmDezMin : begin (* Grad:Min *)
      str (y:(w-6):(d-2),t);
      if copy(t,1,2)='60' then begin
        y:=0.0; x:=x+1.0;
        str (y:(w-6):(d-2),t);
        end;
      if v<0.0 then x:=-x;
      str(x:5:0,s); s:=s+':'+t;
      end;
  fmSekunden : begin (* Grad:Min:Sek *)
      z:=int(y); y:=60.0*frac(y);
      str(y:(w-8):(d-4),t);
      if copy(t,1,2)='60' then begin
        y:=0.0; z:=z+1.0;
        if z>=60.0 then begin
          z:=0.0; x:=x+1.0;
          end;
        str (y:(w-8):(d-4),t);
        end;
      str (z:2:0,s); t:=s+':'+t;
      if v<0.0 then x:=-x;
      str(x:4:0,s); s:=s+':'+t;
      end;
    end;
  i:=pos('.',s); if i>0 then s[i]:=',';
  DegToStr:=s;
  end;

{ ------------------------------------------------------------------- }
(* Integer, Double oder String aus einem String s bis zum nchsten
   Komma lesen
   s wird um den verarbeiteten Teil gekrzt *)
function ReadNxtInt (var s   : String;
                     Del     : char;
                     Default : integer;
                     var err : boolean) : integer;
var
  n    : integer;
  i,ic : integer;
begin
  s:=RemSp(s); i:=pos (Del,s);
  if i=0 then i:=succ(length(s));
  val(copy(s,1,pred(i)),n,ic);
  if ic=0 then ReadNxtInt:=n
  else begin
    ReadNxtInt:=Default;
    err:=true;
    end;
  delete(s,1,i);
  end;

function ReadNxtInt (var s   : String;
                     Del     : char;
                     Default : integer) : integer;
var
  err : boolean;
begin
  Result:=ReadNxtInt(s,Del,Default,err);
  end;

{ ------------------------------------------------------------------- }
function ReadNxtDbl (var s   : String;
                     Del     : char;
                     Default : double;
                     var err : boolean) : double;
var
  i,ic : integer;
  x    : double;
begin
  s:=RemSp(s); i:=pos (Del,s);
  if i=0 then i:=succ(length(s));
  val(SetDecPt(copy(s,1,pred(i))),x,ic);
  if ic=0 then ReadNxtDbl:=x
  else begin
    ReadNxtDbl:=Default;
    err:=true;
    end;
  delete(s,1,i);
  end;

function ReadNxtDbl (var s   : String;
                     Del     : char;
                     Default : double) : double;
var
  err : boolean;
begin
  Result:=ReadNxtDbl(s,Del,Default,err);
  end;

{ ------------------------------------------------------------------- }
function ReadNxtStr (var s   : String;
                     Del     : char) : string;
var
  i : integer;
begin
  if length(s)>0 then begin
    i:=pos (Del,s);
    if i=0 then i:=succ(length(s));
    ReadNxtStr:=copy(s,1,pred(i));
    delete(s,1,i);
    end
  else ReadNxtStr:='';
  end;

{ ------------------------------------------------------------------- }
(* Wort bis zum nchsten Delimiter lesen *)
function ReadNxtWord (var s : string;
                      Dels  : TDelimiters) : string;
var
  i,n : integer;
begin
  n:=length(s);
  if n>0 then begin
    i:=1;
    while (i<=n) and not (s[i] in Dels) do inc(i);
    Result:=copy(s,1,pred(i));
    delete(s,1,pred(i));
    end
  else Result:='';
  end;

{ ------------------------------------------------------------------- }
(* ein Zeichen lesen oder num. Wert #nnn in Zeichen umwandeln *)
function ReadNxtChr (var s   : String;
                     Del     : char;
                     Default : char;
                     var err : boolean) : char;
var
  n,i,ic : integer;
begin
  if (length(s)>0) then begin
    i:=pos (Del,s);
    if (s[1]=ChrID) then begin  (* # *)
      if i=0 then i:=succ(length(s));
      val(copy(s,2,pred(i)),n,ic);
      if ic=0 then ReadNxtChr:=chr(n)
      else begin
        ReadNxtChr:=Default; err:=true;
        end;
      end
    else ReadNxtChr:=s[1];
    delete(s,1,i);
    end
  else begin
    ReadNxtChr:=Default; err:=true;
    end;
  end;

function ReadNxtChr (var s   : String;
                     Del     : char;
                     Default : char): char;
var
  err : boolean;
begin
  Result:=ReadNxtChr(s,Del,Default,err);
  end;

{ ------------------------------------------------------------------- }
(* Testen, ob gltige Integer oder Double Zahl in einem String s
   bis zum nchsten Begrenzer (Del) steht *)
function TestNxtNumber (s       : String;
                        Del     : char ) : boolean;
var
  n,i,ic : integer;
begin
  s:=RemSp(s); i:=pos (Del,s);
  if i=0 then i:=succ(length(s));
  val(copy(s,1,pred(i)),n,ic);
  TestNxtNumber:=ic=0;
  end;

{ ---------------------------------------------------------------- }
(* Dezimaltrenner auf Zeichen setzen, Standard = Punkt *)
procedure SetDecPtChar (Zeichen : char);
begin
  DezTr:=Zeichen;
  end;

{ ---------------------------------------------------------------- }
(* aktuellen Dezimaltrenner lesen *)
function GetDecPtChar : char;
begin
  GetDecPtChar:=DezTr;
  end;

{ ---------------------------------------------------------------- }
(* Tausendermarkierung, Standard = Leerzeichen *)
procedure ThMarkChar (Zeichen : char);
begin
  ThMark:=Zeichen;
  end;

{ ---------------------------------------------------------------- }
(* Integer-Zahl in String umsetzen *)
function StrInt (x : int64;
                 n : integer) : string;
(* n > 0 : Anzahl der Stellen
     = 0 : unformatiert
     < 0 : unformatiert mit Dreiergruppierung *)
var
  s   : string;
  i,j : integer;
begin
  s:=IntToStr(x);
  if n>length(s) then  s:=AddSp(s,n)
  else if n<0 then begin
    i:=succ(length(s));
    j:=i;
    while (j>4) do begin
      dec(j,3); insert (ThMark,s,j);
      end;
    end;
  StrInt:=s;
  end;

{ ---------------------------------------------------------------- }
(* Double-Zahl in String umsetzen *)
function StrDbl (x   : double;
                 n,d : integer) : string;
var
  s : string;
begin
  str (x:n:d,s);
  if DezTr<>Punkt then s:=ReplDecPt (s);
  StrDbl:=s;
  end;

{ ---------------------------------------------------------------- }
(* Integer-Zahl in String mit fhrenden Nullen umsetzen *)
function ZStrInt (x : int64;
                  n : integer) : string;
var
  s : string;
begin
  s:=IntToStr(abs(x));
  if x<0 then begin
    s:=AddChar(s,'0',n-1);
    s:='-'+s;
    end
  else s:=AddChar(s,'0',n);
  ZStrInt:=s;
  end;

{ ---------------------------------------------------------------- }
(* Double-Zahl in String mit fhrenden Nullen umsetzen *)
function ZStrDbl (x   : double;
                  n,d : integer) : string;
var
  s : string;
  i : integer;
begin
  str (abs(x):n:d,s);
  if x<0 then if s[1]=Space then s[1]:='-' else s:='-'+s;
  for i:=1 to n do if s[i]=' ' then s[i]:='0';
  if DezTr<>Punkt then s:=ReplDecPt (s);
  ZStrDbl:=s;
  end;

{ ------------------------------------------------------------------ }
(* Double-Zahl in String in Dreiergruppen umsetzen*)
function NStrDbl (x : double;
                  n,d : integer) : string;
(* Format wie StrDbl *)
var
  s   : string;
  i,j : integer;
begin
  s:=StrDbl(x,n,d);
  i:=pos(DezTr,s);
  if i=0 then i:=succ(length(s));
  j:=i;
  while (j>3) do begin
    dec(j,3); insert (ThMark,s,j);
    inc(i);
    end;
  inc(i,3);
  while (i<length(s)) do begin
    insert(ThMark,s,i); inc(i);
    end;
  NStrDbl:=s;
  end;

{ ---------------------------------------------------------------- }
(* Bytes, je nach Gre in String umsetzen *)
function SizeToStr (Bytes : int64) : string;
var
  v : double;
  n : integer;
begin
  if Bytes<1024 then Result:=StrInt(Bytes,0)+' B'
  else begin
    v:=Bytes/1024;
    if v<1024 then begin
      if v<100 then n:=2 else n:=1;
      Result:=StrDbl(v,0,n)+' kB';
      end
    else begin
      v:=v/1024;
      if v<1024 then begin
        if v<100 then n:=2 else n:=1;
        Result:=StrDbl(v,0,n)+' MB'
        end
      else Result:=StrDbl(v/1024,0,2)+' GB';
      end;
    end;
  end;

{ ---------------------------------------------------------------- }
(* Wert auf Gltigkeit prfen
   true : x <> NoVal *)
function Valid (x,NoVal : double) : boolean;
begin
  Valid:=abs(x-NoVal)>0.1;
  end;

{ ---------------------------------------------------------------- }
(* Dezimalpunkt durch vorgegeben Trenner (SetDecPtChar) ersetzen *)
function ReplDecPt (s : string) : string;
var
  i : byte;
begin
  i:=pos (Punkt,s);
  if i>0 then s[i]:=DezTr;
  ReplDecPt:=s;
  end;

(* aktuellen Dezimaltrenner durch Punkt ersetzen *)
function SetDecPt (s : string) : string;
var
  i : byte;
begin
  i:=pos (DezTr,s);
  if i>0 then s[i]:=Punkt;
  SetDecPt:=s;
  end;

{ ---------------------------------------------------------------- }
(* Cardinal in Hi- oder Lo-Wort zerlegen *)
function HiWord (n : cardinal) : word;
var
  Long : TLongWord absolute n;
begin
  HiWord:=Long.Hi;
  end;

function LoWord (n : cardinal) : word;
var
  Long : TLongWord absolute n;
begin
  LoWord:=Long.Lo;
  end;

(* Cardinal aus Hi- und Lo-Wort zusammensetzen *)
function JoinWord (HiWord,LoWord : word) : cardinal;
var
  Long : TLongWord;
begin
  with Long do begin
    Hi:=HiWord; Lo:=LoWord;
    JoinWord:=LongWord;
    end;
  end;

(* Cardinal aus 4 Bytes zusammensetzen *)
function JoinByte (HHByte,HLByte,LHByte,LLByte : word) : cardinal;
var
  Long : TLongWord;
begin
  with Long do begin
    LoL:=LLbyte; LoH:=LHByte;
    HiL:=HLByte; HiH:=HHByte;
    JoinByte:=LongWord;
    end;
  end;

{ ---------------------------------------------------------------- }
// Convert 16-bit oder 8-Bit number to hex string
function HexByte (b : byte) : string;
const
  HexTab : array [0..15] of char = '0123456789ABCDEF';
begin
  HexByte:=HexTab[b shr 4] + HexTab[b and $f];
  end;

function HexWord (w : word) : string;
begin
  HexWord:=HexByte(hi(w))+HexByte(lo(w));
  end;

//-----------------------------------------------------------------------------
{ QUICKSORT sorts elements in the array "SList" of any record type of
  size "RSize" (in bytes) with indices between 0 and "Count"-1 (both inclusive).
  Note that the QuickSort procedure provides only an "interface" to the program.
  The actual processing takes place in the "Sort" procedure, which executes
  itself recursively.
  "Compare" is a callback function of type "TCompFunction" (see begin of unit).
  Algorithm based on Borland pascal sample "qsort.pas",
  modified J. Rathlev, Nov. 2001 (rathlev@physik.uni-kiel.de) }
//-----------------------------------------------------------------------------
type
  TByteDynArray = array of byte;

procedure QuickSort (var SList;
                     Count,RSize : Integer;
                     Compare     : TCompFunction);
var
  {the  following variables are global to "Sort" to spare memory in recursive calls }
  tmp,ns  : TByteDynArray;
  pl      : PByteArray;
  n,m     : integer;

  procedure Sort(nl,nr: Integer);
  var
    i,j   : integer;
  begin
    n:=RSize*((nl+nr) div 2);
    move (pl^[n],ns[0],RSize);
    i:=nl; j:=nr;
    repeat
      while Compare(pl^[RSize*i],ns[0]) do inc(i);
      while Compare(ns[0],pl^[RSize*j]) do dec(j);
      if i<=j then begin
        n:=RSize*i; m:=RSize*j;
        move (pl^[n],tmp[0],RSize);
        move (pl^[m],pl^[n],RSize);
        move (tmp[0],pl^[m],RSize);
        inc(i); dec(j);
        end;
      until i>j;
    if nl<j then Sort(nl,j);
    if i<nr then Sort(i,nr);
    end;

begin {QuickSort};
  SetLength(Tmp,RSize); SetLength(ns,RSize);
  pl:=@SList;
  Sort(0,Count-1);
  Tmp:=nil; ns:=nil;
  end;

{ ------------------------------------------------------------------- }
(* Erweiterte Druckerangaben *)
(* tatschliche Papierbreite in mm *)
function GetPaperWidth (APrinter : TPrinter) : integer;
begin
  with APrinter do
    Result:=round(25.4*GetDeviceCaps(Handle,PHYSICALWIDTH)/GetDeviceCaps(Handle,LOGPIXELSX));
  end;

(* tatschliche Papierhhe in mm *)
function GetPaperHeight (APrinter : TPrinter) : integer;
begin
  with APrinter do
    Result:=round(25.4*GetDeviceCaps(Handle,PHYSICALHEIGHT)/GetDeviceCaps(Handle,LOGPIXELSX));
  end;

(* nichtdruckbarer Bereich am linken Rand in mm *)
function GetLeftOffset (APrinter : TPrinter) : integer;
begin
  with APrinter do
    Result:=round(25.4*GetDeviceCaps(Handle,PHYSICALOFFSETX)/GetDeviceCaps(Handle,LOGPIXELSX));
  end;

(* nichtdruckbarer Bereich am oberen Rand in mm *)
function GetTopOffset (APrinter : TPrinter) : integer;
begin
  with APrinter do
    Result:=round(25.4*GetDeviceCaps(Handle,PHYSICALOFFSETY)/GetDeviceCaps(Handle,LOGPIXELSX));
  end;

(* nutzbare Papierbreite in mm *)
function GetMaxWidth (APrinter : TPrinter) : integer;
begin
  with APrinter do
    Result := GetDeviceCaps(Handle, HorzSize);
  end;

(* nutzbare Papierhhe in mm *)
function GetMaxHeight (APrinter : TPrinter) : integer;
begin
  with APrinter do
    Result := GetDeviceCaps(Handle, VertSize);
  Sleep(100);
  end;

{ ---------------------------------------------------------------- }
(* Prfen, ob ein Fenster auf den Bildschirm passt, bei Bedarf
   Left, und Top anpassen *)
procedure CheckScreenBounds (Screen         : TScreen;
                             var ALeft,ATop : integer;
                             AWidth,AHeight : integer);
begin
  with Screen do begin
    if ALeft+AWidth>Width then ALeft:=Width-AWidth-20;
    if ATop+AHeight>Height then ATop:=Height-AHeight-20;
    end;
  end;

{ ---------------------------------------------------------------- }
// neuer Message-Dialog mit Positionsprfung
function MessageDialog(const Title,Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; Pos : TPoint) : integer;
begin
  with CreateMessageDialog(Msg, DlgType, Buttons) do
    try
      with Pos do begin
        if (Y < 0) or (X < 0) then Position := poScreenCenter
        else begin
          if X<0 then X:=Left;
          if Y<0 then Y:=Top;
          CheckScreenBounds(Screen,x,y,Width,Height);
          Left:=x; Top:=y;
          end;
        end;
      if length(Title)>0 then Caption:=Title;
      Result := ShowModal;
    finally
      Free;
    end;
  end;

{ ---------------------------------------------------------------- }
// Besttigung in Bildschirmmitte (X<0) oder an Position X,Y
function ConfirmDialog (const Title,Msg  : string; Pos : TPoint) : boolean;
var
  s : string;
begin
  if length(Title)=0 then s:='Besttigung' else s:=Title;
  Result:=MessageDialog (Title,Msg,mtConfirmation,[mbYes,mbNo],Pos)=mrYes;
  end;

// Information ausgeben
procedure InfoDialog (const Title,Msg : string; Pos : TPoint);
var
  s : string;
begin
  if length(Title)=0 then s:='Information' else s:=Title;
  MessageDialog (Title,Msg,mtInformation,[mbOK],Pos);
  end;

// Fehlermeldung ausgeben
procedure ErrorDialog (const Title,Msg : string; Pos : TPoint);
var
  s : string;
begin
  if length(Title)=0 then s:='Fehler' else s:=Title;
  MessageDialog (Title,Msg,mtError,[mbOK],Pos);
  end;

{ ------------------------------------------------------------------- }
// get current cursor position
function CursorPos : TPoint;
var
  pos : TPoint;
begin
  GetCursorPos(pos);
  Result:=pos;
  end;

{ ------------------------------------------------------------------- }
// get current cursor position, add Offset 
function CursorOffsetPos (Ofs : TPoint): TPoint;
var
  pos : TPoint;
begin
  GetCursorPos(pos);
  with pos do begin
    x:=x+Ofs.x; y:=y+Ofs.y;
    end;
  Result:=pos;
  end;

{ ------------------------------------------------------------------- }
// History list management
const
  iniHist = 'History';

procedure LoadHistory (IniFile : TIniFile; Section : string; History : TStrings);
var
  i : integer;
  s : string;
begin
  with IniFile do begin
    if SectionExists(Section) then begin
      History.Clear;
      for i:=0 to MaxHist-1 do begin
        s:=ReadString(Section,iniHist+IntToStr(i),'');
        if s<>'' then History.Add(s);
        end;
      end;
    end;
  end;

procedure SaveHistory (IniFile : TIniFile; Section : string; History : TStrings);
var
  i : integer;
begin
  with IniFile do begin
    EraseSection (Section);
    with History do for i:=0 to Count-1 do
      WriteString(Section,iniHist+IntToStr(i),Strings[i]);
    end;
  end;

procedure AddToHistory (History : TStrings; hs : string);
var
  n : integer;
begin
  if length(hs)>0 then with History do begin
    n:=IndexOf(hs);
    if n<0 then begin
      if Count>=MaxHist then Delete (Count-1);
      Insert (0,hs);
      end
    else Move (n,0);
    end;
  end;

procedure RemoveFromHistory (History : TStrings; hs : string);
var
  n : integer;
begin
  if length(hs)>0 then with History do begin
    n:=IndexOf(hs);
    if n>=0 then Delete (n);
    end;
  end;

//-----------------------------------------------------------------------------
procedure FreeListObjects (Liste : TStrings);
var
  i : integer;
begin
  with Liste do begin
    for i:=0 to Count-1 do if assigned(Objects[i]) then begin
      Objects[i].Free; Objects[i]:=nil;
      end;
    end;
  end;

//-----------------------------------------------------------------------------
// Tastaturpuffer lschen
function ClearKeyboardBuffer : Integer;
var
   Msg: TMsg;
begin
  Result := 0;
  while PeekMessage(Msg,0,WM_KEYFIRST,WM_KEYLAST,PM_REMOVE) do inc(Result);
  end;

{ ------------------------------------------------------------------- }
(* Verzgerungszeit unter Verwendung der Windows Waitable Timer Objects
   Wait: warten bis Zeit in ms abgelaufen ist (min. Zeit 100 ns) *)

constructor TWaitTimer.Create;
begin
  inherited Create;
  FTimer:=CreateWaitableTimer(nil,true,'WaitableTimer');
  end;

destructor TWaitTimer.Destroy;
begin
  CloseHandle(FTimer);
  end;

procedure TWaitTimer.Wait (DelayTime : double);
var
  d : TLargeInteger;
begin
  d:=-round(10000*DelayTime);
  SetWaitableTimer(FTimer,d,0,nil,nil,false);
  repeat until WaitForSingleObject(FTimer,INFINITE)=WAIT_OBJECT_0;
  end;

{ ------------------------------------------------------------------- }
begin
  DezTr:=DecimalSeparator;
end.
