(* Starten eines beliebigen (auch DOS) Programms aus Windows heraus
   Ein Parameter (z.B. ein Dateiname) wird im Dialog abgefragt
   J. Rathlev, Jan. 1999
     letzte nderung: Aug. 2002

   Vers. 2.3: Die Kommandozeilenparameter knnen mit einem Editor in
              die Ini-Datei eingegeben werden. Sie sind nur wirksam, wenn
              keine Kommandozeile gefunden wird.
     *)
     
(* Kommandozeile:
   STPROG <EXE-File> [/P<Prompt>] [/E<Ext>] [/O<Options>] [/L<Lang>] [@ParFile]
     EXE-File  : auszufhrendes Programm (auch COM oder BAT)
     Ext       : zustzlicher Button fr die Eingabe einer
                 Standarderweiterung <Ext> fr den abgefragten Parameter
     Prompt    : Prompt im Dialogfenster
     Options   : weitere fest eingestellte Startoptionen fr das
                 auszufhrende Programm
                 Leerzeichen wird durch '_', '/' durch '%' ersetzt
     Lang      : D (deutsch) oder E (englisch)
     @ParFile  : enthlt zeilenweise alle Parameter
                 *)

unit Stprmain;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls, Buttons, IniFiles,
  JrUtils, HListBox, FiltDlg, XPMan;

const
  Vers = '2.31';

  Deutsch = 1;
  English = 2;
  LText : array [1..12,1..2] of string =
    (('Programm-Starter V. '+Vers+#13+
       '(c) 2001 Dr. J. Rathlev - Uni-Kiel'#13+
       'Aufruf: PRLAUNCH <EXE-Datei> [/Lx] [/Px] [/E[x]] [/Ox]'#13+
       '  /Lx  : Sprache (D,E)'#13+
       '  /Px  : Hinweis'#13+
       '  /E   : einstellbare Dateifilter'#13+
       '  /Ex  : feste Standard-Erweiterung fr Dateinamen'#13+
       '  /Ox  : Optionen fr aufzurufendes Programm'#13+
       '         (ersetze Leerzeichen durch _ und / durch %)',
       'Program-Launcher V. '+Vers+#13+
       '(c) 2001 Dr. J. Rathlev - Uni-Kiel'#13+
       'Calling: PRLAUNCH <EXE-File> [/Lx] [/Px] [/E[x]] [/Ox]'#13+
       '  /Lx  : Language (D,E)'#13+
       '  /Px  : Prompt for query'#13+
       '  /E   : Selectable file filter'#13+
       '  /Ex  : Default file extension'#13+
       '  /Ox  : Options for called program'#13+
       '         (replace space by _ and / by %)'),
     (' nicht gefunden!',' not found!'),
     ('Aufruf: ','Calling: '),
     ('Typ ','Type '),
     ('Standardeinstellung fr Typ','Default type'),
     ('Dateinamenerweiterung:','File extension:'),
     ('Datei whlen','Select file'),
     ('Vorgabe','Preselection'),
     ('Auswahl fr Dateifilter oder Standarderweiterung','Select file filter or default extension'),
     ('Abbruch','Cancel'),
     ('|alle|*.*','|all|*.*'),
     ('Datei-Filter','File-Filter')
      );

type
  TStPrForm = class(TForm)
    TextLabel: TLabel;
    TextBevel: TBevel;
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    TextEdit: THistoryCombo;
    ErwBtn: TButton;
    BrowseBtn: TSpeedButton;
    OpenDialog: TOpenDialog;
    InfoBtn: TSpeedButton;
    XPManifest1: TXPManifest;
    procedure StPrFormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure CancelBtnClick(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure ErwBtnClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure BrowseBtnClick(Sender: TObject);
    procedure InfoBtnClick(Sender: TObject);
  private
    { Private-Deklarationen }
    Exename,ExePar,
    StdExt,PrLName,
    IniName,IniSekt : string;
    ShowErwBtn      : boolean;
    Lang            : word;
  public
    { Public-Deklarationen }
  end;

var
  StPrForm: TStPrForm;

implementation

{$R *.DFM}

uses WinDesktop, ShlObj;

const
  IniCfg = 'Config';
  IniLang = 'Language';
  IniPrompt = 'Prompt';
  IniExt = 'Extension';
  IniExe = 'Program';
  IniOpt = 'Options';
  IniLast = 'LastFile';
  IniFilter = 'Filter';
  IniFilterCount = 'FilterCount';

(* Kommandozeile auswerten , INI einlesen *)
procedure TStPrForm.StPrFormCreate(Sender: TObject);
var
  IniFile : TIniFile;
  i,j     : word;
  s       : string;

  (* Optionen auswerten *)
  procedure ReadOptions (s : string);
  var
    c       : char;
    i       : word;
  begin
    c:=UpCase(s[2]); delete(s,1,2);
    case c of
    'L' : case UpCase(s[1]) of
          'D' : Lang:=Deutsch;
          'E' : Lang:=English;
            end;
    'P' : TextLabel.Caption:=s;
    'E' : begin
          ShowErwBtn:=true;
          if length(s)>0 then StdExt:=OEMUpString(s);
          end;
    'O' : begin
          for i:=1 to length(s) do begin
            if s[i]='_' then s[i]:=' ';
            if s[i]='%' then s[i]:='/';
            end;
          ExePar:=s;
          end;
      end;
    end;

  (* Optionen aus Parameter-Daei lesen *)
  procedure ReadParFile (FName : string);
  var
    s       : string;
    ft      : TextFile;
  begin
    delete(FName,1,1);
    if FileExists(FName) then begin
      AssignFile(ft,FName);
      Reset (ft);
      while not Eof(ft) do begin
        readln(ft,s);
        if s[1]='/' then ReadOptions (s)
        else ExeName:=Erweiter('',s,'exe');
        end;
      CloseFile(ft);
      end
    else begin
      MessageDlg (FName+LText[2,Lang],mtError,[mbOK],0);
      halt(0);
      end;
    end;

begin
  ShowErwBtn:=false; StdExt:=''; Lang:=1;
  TextLabel.Caption:='Parameter:'; ExePar:='';
  IniName:=Erweiter(GetDesktopFolder(CSIDL_APPDATA),ExtractFileName(DelExt(Application.ExeName)),'ini');
  IniFile:=TIniFile.Create (IniName);
  if ParamCount=0 then with IniFile do begin (* Konfiguration aus INI-Datei holen*)
    Exename:=ReadString(IniCfg,IniExe,'');
    i:=ReadInteger(IniCfg,IniLang,Lang);
    if (i>0) and (i<3) then Lang:=i;
    TextLabel.Caption:=ReadString(IniCfg,IniPrompt,'Parameter:');
    ShowErwBtn:=ReadBool(IniCfg,IniFilter,ShowErwBtn);
    StdExt:=OEMUpString(ReadString(IniCfg,IniExt,StdExt));
    ExePar:=ReadString(IniCfg,IniOpt,ExePar);
    end
  else begin
    for i:=1 to ParamCount do begin (* Befehlszeile auswerten *)
      s:=ParamStr(i);
      if s[1]='/' then ReadOptions (s)
      else if s[1]='@' then ReadParFile (s)
      else ExeName:=Erweiter('',s,'EXE');
      end;
    end;
  if length(ExeName)=0 then begin
    MessageDlg(LText[1,Lang]+#13#13+LText[1,2],
             mtInformation,[mbOK],0);
    halt(0);
    end;
  if not FileExists(ExeName) then begin
    MessageDlg (ExeName+LText[2,Lang],mtError,[mbOK],0);
    halt(0);
    end;
  IniSekt:=NewExt(ExtractFileName(ExeName),'');
  with IniFile do begin  (* Filter einlesen *)
    PrlName:=ReadString (IniSekt,iniLast,'');
    if length(StdExt)=0 then  begin
      j:=ReadInteger (IniSekt,IniFilterCount,0);
      s:='';
      if j>0 then for i:=0 to pred(j) do
        s:=s+ReadString(IniSekt,IniFilter+ZStrInt(i,2),'')+'|';
      if length(s)>0 then delete (s,length(s),1)
      else s:=LText[8,Lang]+' (*.'+StdExt+')|*.'+StdExt+LText[11,Lang];
      OpenDialog.Filter:=s;
      end
    else OpenDialog.Filter:=LText[8,Lang]+' (*.'+StdExt+')|*.'+StdExt+LText[11,Lang];
    Free;
    end;
  with TextEdit do begin
    IniFileName:=IniName;
    IniSection:=IniSekt;
    Loaded;
    end;
  ErwBtn.Hint:=LText[9,Lang];
  if ShowErwBtn then if length(StdExt)>0 then ErwBtn.Caption:=LText[4,Lang]+StdExt
  else StdExt:='';
  CancelBtn.Caption:=LText[10,Lang];
  end;

procedure TStPrForm.FormClose(Sender: TObject; var Action: TCloseAction);
var
  IniFile : TIniFile;
  i,j     : integer;
  s,t     : string;
begin
  IniFile:=TIniFile.Create (IniName);
  with IniFile do begin
    if PrlName<>'' then WriteString (IniSekt,iniLast,PrlName);
    if length(StdExt)=0 then begin
      j:=0;
      s:=OpenDialog.Filter;
      if length(s)>0 then repeat
        i:=pos('|',s); t:=copy(s,1,i); delete (s,1,i);
        if length(s)>0 then begin
          i:=pos('|',s);
          if i=0 then i:=succ(length(s));
          t:=t+copy(s,1,pred(i)); delete (s,1,i);
          end;
        WriteString(iniSekt,IniFilter+ZStrint(j,2),t);
        inc(j);
        until length(s)=0;
      WriteInteger (IniSekt,IniFilterCount,j);
      end;
    Free;
    end;
end;

procedure TStPrForm.FormShow(Sender: TObject);
var
  d : integer;
begin
  Caption:=LText[3,Lang]+ExeName;
  with TextLabel do TextEdit.Left:=Left+Width+10;
  with TextEdit do d:=Width+Left;
  TextEdit.text:=PrlName;
  if ShowErwBtn then with BrowseBtn do begin
    Left:=d+5;
    d:=d+Width;
    end;
  inc(d,10);
  with TextBevel do Width:=d-Left;
  with CancelBtn do Left:=d-Width;
  with OKBtn do Left:=CancelBtn.Left-65;
  with InfoBtn do Left:=OKBtn.Left-30;
  ClientWidth:=TextBevel.Width+10;
  ErwBtn.Visible:=ShowErwBtn;
  BrowseBtn.Visible:=ShowErwBtn;
end;

procedure TStPrForm.CancelBtnClick(Sender: TObject);
begin
  Close;
end;

procedure TStPrForm.OKBtnClick(Sender: TObject);
var
  CmdLine : string;
begin
  with TextEdit do AddItem(Text);
  if FileExists(ExeName) then begin
    PrlName:=TextEdit.Text;
    CmdLine:=ExeName+' '+ExePar+' '+Erweiter('',TextEdit.Text,StdExt);
    WinExec (pchar(CmdLine),SW_SHOW);
    end
  else MessageDlg (ExeName+LText[2,Lang],mtError,[mbOK],0);
  Close;
  end;

procedure TStPrForm.ErwBtnClick(Sender: TObject);
var
  s : string;
begin
  if length(StdExt)>0 then begin
    s:=StdExt;
    if InputQuery (LText[5,Lang],LText[6,Lang],s) then begin
      StdExt:=OEMUpString(copy(s,1,3));
      ErwBtn.Caption:=LText[4,Lang]+StdExt;
      end;
    end
  else begin
    s:=OpenDialog.Filter;
    if FilterDialog.Execute (LText[12,Lang],s) then OpenDialog.Filter:=s;
    end;
  end;

procedure TStPrForm.BrowseBtnClick(Sender: TObject);
begin
  with OpenDialog do begin
    InitialDir:=ExtractFilePath(TextEdit.Text);
    DefaultExt:=StdExt; Title:=LText[7,Lang];
    if Execute then TextEdit.Text:=FileName;
    end;
  end;

procedure TStPrForm.InfoBtnClick(Sender: TObject);
begin
  MessageDlg(LText[1,Lang],
             mtInformation,[mbOK],0);
  end;

end.

